@@ -1,24 +1,25 @@
-/** *****************************************************************************
- * Licensed Materials - Property of Red Hat, Inc.
- * Copyright (c) 2020 Red Hat, Inc.
- ****************************************************************************** */
+// Copyright (c) 2020 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 /// <reference types="cypress" />
 
 export const targetResource = data => {
   const name = data.name;
   const config = data.config;
+  const clusterName = Cypress.env("managedCluster");
   const kubeconfigs = Cypress.env("KUBE_CONFIG");
   if (kubeconfigs) {
-    kubeconfigs.forEach(kubeconfig => {
-      cy.log(`cluster - ${kubeconfig}`);
-      for (const [key, value] of Object.entries(config)) {
-        cy.log(`instance-${key}`);
-        !value.deployment.local
-          ? subscription(key, name, kubeconfig, "contain")
-          : cy.log(`${name} has been deployed locally`);
-      }
-    });
+    const kubeconfig =
+      kubeconfigs.length > 1
+        ? kubeconfigs.find(kube => kube.includes(clusterName))
+        : kubeconfigs[0];
+    cy.log(`cluster - ${kubeconfig}`);
+    for (const [key, value] of Object.entries(config)) {
+      cy.log(`instance-${key}`);
+      !value.deployment.local
+        ? subscription(key, name, kubeconfig, "contain")
+        : cy.log(`${name} has been deployed locally`);
+    }
   } else {
     cy.log(
       `skipping validating resource on managed cluster as no kubeconfig is provided`
@@ -44,9 +45,39 @@ export const apiResources = (type, data, returnType) => {
   }
 };
 
-export const checkExistingUrls = (css1, value1, css2, value2, url) => {
-  getSavedPathname().then(({ urllist }) => {
-    if (!urllist.includes(url)) {
+// check if this is an existing channel
+// for new channels enter the credentials, if set
+// for existing channel select the full info, which is 'path [channelNS/channelName]
+export const checkExistingUrls = (
+  css1,
+  value1,
+  css2,
+  value2,
+  repoFieldId,
+  url
+) => {
+  getSavedPathname().then(pathnames => {
+    const { urllist } = pathnames;
+    let urlPathForChannel = url;
+    let existingChannel = false;
+
+    urllist.forEach(channel => {
+      // should be in this form : 'path [channelNS/channelName]'
+      const channelDataList = channel.split(" ");
+      if (!existingChannel && channelDataList[0].includes(url)) {
+        cy.log(`credentials have been saved for url - ${url}`);
+        urlPathForChannel = channel;
+        existingChannel = true;
+      }
+    });
+
+    cy
+      .get(repoFieldId, { timeout: 20 * 1000 })
+      .type(urlPathForChannel, { timeout: 50 * 1000 })
+      .blur();
+
+    if (!existingChannel) {
+      cy.log(`New channel, set credentials if this is a private repo - ${url}`);
       if (value1 && value2) {
         cy.get(css1, { timeout: 20 * 1000 }).paste(value1, {
           log: false,
@@ -59,24 +90,37 @@ export const checkExistingUrls = (css1, value1, css2, value2, url) => {
           failOnNonZeroExit: false
         });
       }
-    } else {
-      cy.log(`credentials have been saved for url - ${url}`);
+      urllist.push(url);
     }
   });
 };
 
+let savedPathnames = null;
 export const getSavedPathname = () => {
   // returns a list of existing pathnames
-  return cy
-    .exec(`oc get channels -o=jsonpath='{.items[*].spec.pathname}' -A`, {
-      timeout: 50 * 1000
-    })
-    .then(result => {
-      const urllist = result.stdout.split(" ").filter(i => i);
-      return {
-        urllist: urllist
-      };
-    });
+  if (!savedPathnames) {
+    const pathForChannel = [];
+    return cy
+      .exec(
+        `oc get channels -A -o custom-columns='pathname:.spec.pathname,namespace:.metadata.namespace,name:.metadata.name' --no-headers`,
+        {
+          timeout: 50 * 1000
+        }
+      )
+      .then(({ stdout }) => {
+        const urllist = stdout.split("\n").map(channel => channel.split(/ +/));
+        urllist.forEach(channel => {
+          // should be 'path channelNS channelName'
+          pathForChannel.push(`${channel[0]} [${channel[1]}/${channel[2]}]`);
+        });
+        savedPathnames = {
+          urllist: pathForChannel
+        };
+        return savedPathnames;
+      });
+  }
+
+  return cy.wrap(savedPathnames);
 };
 
 export const channelsInformation = (name, key, namespace = "default") => {
@@ -186,7 +230,7 @@ export const getNumberOfManagedClusters = () => {
   cy
     .exec(
       `oc get managedclusters -o custom-columns='name:.metadata.name,available:.status.conditions[?(@.type=="ManagedClusterConditionAvailable")].status,vendor:.metadata.labels.vendor' --no-headers`,
-      { failOnNonZeroExit: false }
+      { failOnNonZeroExit: true }
     )
     .then(({ stdout }) => {
       const clusters = stdout.split("\n").map(cluster => cluster.split(/ +/));
@@ -207,7 +251,9 @@ export const getManagedClusterName = () => {
   cy
     .exec(
       `oc get managedclusters -o custom-columns='name:.metadata.name,available:.status.conditions[?(@.type=="ManagedClusterConditionAvailable")].status,vendor:.metadata.labels.vendor' --no-headers`,
-      { failOnNonZeroExit: false }
+      {
+        failOnNonZeroExit: true
+      }
     )
     .then(({ stdout }) => {
       const clusters = stdout.split("\n").map(cluster => cluster.split(/ +/));
@@ -251,7 +297,8 @@ export const deleteNamespaceTarget = (name, kubeconfig) => {
           if ((stdout || stderr).includes("not found") === false) {
             cy
               .exec(
-                `oc ${managedCluster} ${managedCluster} delete ns ${name}-ns`
+                `oc ${managedCluster} ${managedCluster} delete ns ${name}-ns`,
+                { failOnNonZeroExit: false }
               )
               .its("stdout")
               .should("contain", `${name}-ns`);
